--[[============================================================================
main.lua
============================================================================]]--

-- Placeholder for the dialog
local dialog = nil

-- Placeholder to expose the ViewBuilder outside the show_dialog() function
local vb = nil

-- Reload the script whenever this file is saved. 
-- Additionally, execute the attached function.
_AUTO_RELOAD_DEBUG = function()
  
end

-- Read from the manifest.xml file.
class "RenoiseScriptingTool" (renoise.Document.DocumentNode)
  function RenoiseScriptingTool:__init()    
    renoise.Document.DocumentNode.__init(self) 
    self:add_property("Name", "Untitled Tool")
    self:add_property("Id", "Unknown Id")
  end

local manifest = RenoiseScriptingTool()
local ok,err = manifest:load_from("manifest.xml")
local tool_name = manifest:property("Name").value
local tool_id = manifest:property("Id").value

prefs = renoise.Document.create {
  time = 1.0
}
renoise.tool().preferences = prefs



--------------------------------------------------------------------------------
-- GUI
--------------------------------------------------------------------------------

local function show_dialog()

  if dialog and dialog.visible then
    dialog:show()
    return
  end
  
  vb = renoise.ViewBuilder()
  
  local max_time = 1.0
  local sample = renoise.song().selected_instrument:sample(1)
  local buffer = sample.sample_buffer
  if (buffer.has_sample_data) then
    max_time = (buffer.number_of_frames / buffer.sample_rate) * 1000
  end

  local vb_time = vb:valuefield {
    width = 150,
    min = 0,
    max = max_time,
    value = prefs.time.value,
    notifier = function(value) 
      prefs.time.value = value 
    end,
  }


  local content = vb:column {
    margin = 10,
    spacing = 10,
    vb:text {
      text = "Slice position in milliseconds..."
    },
    vb:column {
      style = "border",
      margin = 5,
      vb_time
    },
    vb:button {
      text = 'Add Slice',
      width = 160,
      height = 40,
      notifier = function()
        local pos = math.floor(prefs.time.value * (buffer.sample_rate / 1000))
        sample:insert_slice_marker(pos)
        dialog:close()
      end
    }
  } 
  
  dialog = renoise.app():show_custom_dialog(tool_name, content)  
  
end


--------------------------------------------------------------------------------
-- Menu entries
--------------------------------------------------------------------------------

renoise.tool():add_menu_entry {
  name = "Sample Editor:Slices:"..tool_name.."...",
  invoke = show_dialog  
}


--------------------------------------------------------------------------------
-- Key Binding
--------------------------------------------------------------------------------

renoise.tool():add_keybinding {
  name = "Sample Editor:Slices:" .. tool_name.."...",
  invoke = show_dialog
}
